/*                
 *                        Licensing Notification
 *
 * Permission to use, reproduce, prepare derivative works, and to redistribute
 * to others this software and its documentation is hereby granted, provided
 * that this notice is retained thereon and on all copies or modifications. UChicago
 * Argonne, LLC, Tech-X Corporation, and all other contributors make no representations
 * as to the suitability and operability of this software for any purpose. It is
 * provided "as is" without express or implied warranty.
 *
 */

#ifndef TX_HYB_MATRIX_H
#define TX_HYB_MATRIX_H

#include <cusp/hyb_matrix.h>
#include <tx_common.h>
// -----------------------------------------------------------------------------

/*
	Kernel function is based on CUSP library's spmv_hyb_kernel
	function which is licensed under the Apache 2.0 license.  Here
	is the license information for that part of the code.

 *  Copyright 2008-2012 NVIDIA Corporation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.

*/

// -----------------------------------------------------------------------------

#define CUSPMATRIXHYB cusp::hyb_matrix<PetscInt,PetscScalar,cusp::device_memory>

/***********************/
/*   Hybrid Matrix     */
/***********************/
template <typename index_type, typename value_type>
class HybMatrix {
public:
  typedef CUSPMATRIXHYB MatrixType;
 
  HybMatrix() {
  }

  /* Destructor */
  virtual ~HybMatrix() {
  }
 
  /* Set the Matrix explicitly on the device*/
  void setMatrix(const cusp::csr_matrix<index_type,value_type,cusp::host_memory>& matCPU) {
    A = matCPU;
  }  

  /* Set the Matrix */
  void setMatrix(const PetscInt numRows, const PetscInt numCols, const PetscInt nz,
		 const PetscInt * row_offsets, const PetscInt * column_indices,
		 const PetscScalar * values) {

    // create a temporary 
    cusp::csr_matrix<PetscInt,PetscScalar,cusp::host_memory> B(numRows,numCols,nz);    
    B.row_offsets.assign(row_offsets, row_offsets+numRows+1);
    B.column_indices.assign(column_indices, column_indices+nz);
    B.values.assign(values, values+nz);
    // Let CUSP do the conversion
    A = B;
  }

  /* Set the Indices For cprow==true (compressed row) */
  void setCPRowIndices(const index_type * indices, const PetscInt m) {
    // does nothing as of now
  }

  /* Set the inode data for when inode.use==true (inodes) */
  virtual void setInodeData(const PetscInt * inodesCPU, const int inodeCountPlusOne, const int nnzPerRowMaxCPU,
                            const int nodeMaxCPU, const int nodeCountCPU) {
    // does nothing as of now
  }

  /* SpMV */
  int multiply(const cudaStream_t stream, const value_type *x, value_type *y) {
    return 0;
  }

  /* SpMV Add*/
  int multiplyAdd(const cudaStream_t stream, const value_type *x, value_type *y) {
    return 0;
  }

private:
  MatrixType A; /* container for the matrix */
};

#endif // TX_HYB_MATRIX_H
